parts/: parts[integrand_, x_, u_, substitution_] := 
    {newintegrand = Integrate[integrand/substitution, x]*D[substitution, x],
    u -> substitution, v -> Integrate[integrand/substitution, x]}    

parts::usage = "parts[integrand, x, u, substitution] applies integration
by parts to the original integrand (in terms of x) with u = substitution.
It outputs a list of three elements: the first member is the integrand u
dv (newintegrand); the second indicates the choice of u; the third
indicates the value of v."

transform/: transform[integrand_, x_, u_, substitution_] :=
    {newintegrand = integrand/D[substitution, x] /. substitution -> u,
    u == substitution}

transform::usage = "transform[integrand, x, u, substitution] applies the
change of variables u = substituion (in terms of x) to the original
integrand (in terms of x). It outputs a list of two elements: the first
element is the new integrand (newintegrand) in terms of u (and possibly
x); the second element is an equation giving the substitution u in terms of x."


inversesub/: inversesub[integrand_, x_, z_, substitution_] := 
    {newintegrand = (integrand /. x -> substitution)*D[substitution, z],
        Solve[x == substitution, z]}

inversesub::usage = "inversesub[integrand, x, z, substitution] applies the
change of variables x = substitution (in terms of z) to the original
integrand (in terms of x). It outputs a list of two elements: the first
element is the new integrand (newintegrand) in terms of z; the second
element is a list of solutions to the equation x == substitution, giving z
as one or more functions of x." 


completesquare/: completesquare[quadratic_, x_] :=
     Coefficient[quadratic, x^2]*
    (x + Coefficient[quadratic, x]/(2*Coefficient[quadratic, x^2]))^2 +
     Coefficient[quadratic, x^2]*
    ((4*Coefficient[quadratic, x^2]*(quadratic /. x -> 0) -
        Coefficient[quadratic, x]^2)/(4*Coefficient[quadratic, x^2]^2))

completesquare::usage = "completesquare[quadratic, x] completes the square
of the quadratic function of x."

