bisect/: bisect[f_][{a_, b_}] := 
   If[N[f[a]*f[b]] > 0, {a, b},
   If[N[f[a]*f[(a + b)/2]] <= 0, N[{a, (a + b)/2}], N[{(a + b)/2, b}]]]
 
bisect::usage = "bisect[f][{a, b}] determines which half of the interval
{a, b} contains a solution to the equation f[x] == 0. It is assumed that f
is continuous on {a, b}. If f[a] and f[b] have the same sign, then the
output is the original interval."

tangent/: tangent[f_, a_][x_] := f'[a]*(x - a) + f[a]


normal/: normal[f_, a_][x_] := (a - x)/f'[a] + f[a]


tangentplot/: tangentplot[f_, a_, {b_, c_}] := 
   Plot[{f[x], tangent[f, a][x]}, {x, b, c}]
 

normalplot/: normalplot[f_, a_, {b_, c_}] :=
   Plot[{f[x], normal[f, a][x]}, {x, b, c}, AspectRatio -> Automatic]


errorplot/: errorplot[f_, a_, {b_, c_}] :=
   Plot[Abs[f[x] - tangent[f, a][x]], {x, b, c}, PlotRange -> All]

errorplot::usage = "errorplot[f, a, {b, c}] plots the absolute value of
the difference of f and the tangent to f at x = a, for each x in the
interval {b, c}."

iteration/: iteration[f_][x_] := x - f[x]/f'[x]


newtonsmethod/: 
  newtonsmethod[f_, a_, {b_, c_}][n_] := 
   (one = 
    Block[{x, i}, Plot[{f[x], tangent[f, Nest[iteration[f], N[a], n]][x]},
 {x, First[Union[{b}, Table[Nest[iteration[f], N[a], i], {i, 0, n +1}]]],
  Last[Union[{c}, Table[Nest[iteration[f], N[a], i], {i, 0, n + 1}]]]},
    DisplayFunction -> Identity]]; 
    two = 
    Block[{i}, ListPlot[Table[{Nest[iteration[f], N[a], i], 0}, {i, 0, n + 1}],
      DisplayFunction -> Identity]];
    three = 
    Block[{i}, ListPlot[Table[{Nest[iteration[f], N[a], i],
         f[Nest[iteration[f], N[a], i]]}, {i, 0, n}],
             DisplayFunction -> Identity]];
       Show[one, two, three, PlotLabel -> 
         If[n == 0, If[a == 0, "0 = initial guess", If[a == 1,
        "1 = initial guess", a " = initial guess"]],
           {(n + 1) " iterations of Newton's method completed",
            N[Nest[iteration[f], a, n + 1]] " = approximation"} ],
              DisplayFunction -> $DisplayFunction])
 
newtonsmethod::usage = "newtonsmethod[f, a, {b, c}][n] shows Newton's
method being used to generate the (n + 1)st approximation to the solution
of the equation f[x] == 0 from the nth approximation, starting with
initial guess x0 = a. All previous approximations are shown along the
x-axis, and the value of the current approximation is indicated in the
PlotLabel. The interval {b, c} is the initial x-interval used for the
plot; however the x-interval will adjust automatically, if necessary, to
include all approximations x0, x1, ... , xn, x(n + 1)."


fixedpoint/: fixedpoint[g_, a_, {b_, c_}][n_] := 
   (one = 
    Block[{x, i}, Plot[{g[x], x}, {x, b, c},
        AspectRatio -> Automatic, PlotRange -> {{b, c}, {b, c}},
        DisplayFunction -> Identity]]; 
    two = 
    Block[{i}, ListPlot[Table[{Nest[g, N[a], i], 0}, {i, 0, n + 1}],
                DisplayFunction -> Identity]]; 
    three = 
    Block[{i}, ListPlot[Table[{Nest[g, N[a], i], Nest[g, N[a], i]},
           {i, 0, n + 1}], DisplayFunction -> Identity]]; 
    four = 
    Block[{i}, ListPlot[Table[{Nest[g, N[a], i], Nest[g, N[a], i + 1]},
           {i, 0, n + 1}], DisplayFunction -> Identity]];
    five = 
    Block[{i}, ListPlot[Table[{Nest[g, N[a], Floor[i/2]],
        Mod[i + 1, 2]*Nest[g, N[a], Floor[i/2]] +
        Mod[i, 2]*g[Nest[g, N[a], Floor[i/2]]]}, {i, 0, 2 n + 3}],
        PlotJoined -> True, DisplayFunction -> Identity]];
    Show[one, two, three, four, five,
        PlotLabel -> If[n == 0,
             {"beginning fixed point iteration",
        If[a == 0, "0 = initial guess", If[a == 1,
        "1 = initial guess", a " = initial guess"]]},
   {(n + 1) " iterations completed", N[Nest[g, a, n + 1]] " = approximation"}],
                DisplayFunction -> $DisplayFunction])

fixedpoint::usage = "fixedpoint[f, a, {b, c}][n] performs (n + 1)
iterations of the Fixed Point iteration method for approximating solutions
to the equation f[x] == x, starting with x0 = a. The output plots f and
the identity function, y = x, on the interaval {b, c}; shows all
successive approximations x0, x1, ... , xn, x(n + 1) along the x-axis;
indicates the value of the current approximation in the PlotLabel; and
shows the `cobweb' of points homing in on the fixed point."


