riemann/: riemann[f_, {a_, b_}, n_, choice_:1] := 
   N[(b - a)*Sum[f[(1 - choice)*(a + (i-1)*(b - a)/n) + 
       choice*(a + i*(b - a)/n)], {i, 1, n}]/n]

riemann::usage = "riemann[f, {a, b}, n, choice] calculates the numerical
value of the Riemann sum obtained by dividing the interval {a, b} into n equal
subintervals and evaluating f at the chosen point of each subinterval. The
chosen point is determined by a number between 0 and 1, ranging from the
left end point to the right end point of each subinterval. The default
choice is 1, ie. choosing the right end point of each subinterval."
 

er/: er[f_, {a_, b_}, n_, choice_:1] := 
    Abs[NIntegrate[f[x], {x, a, b}] - riemann[f, {a, b}, n, choice]]

er::usage = "er[f, {a, b}, n, choice] computes the absolute value of the
difference between the value of the definite integral of f on the interval
{a, b} and the value of the Riemann sum approximation riemann[f, {a, b},
n, choice]."
 

definitetransform/: 
  definitetransform[integrand_, {x_, a_, b_}, u_, substitution_] := 
   {newintegrand = integrand/D[substitution, x] /. substitution -> u, 
    u == substitution, {substitution /. x -> a, substitution /. x -> b}}
 
definitetransform::usage = "definitetransform[integrand, {x, a, b}, u,
substitution] applies the change of variables u = substitution (in terms
of x) to the definite integral of the given integrand (in
terms of x) on the interval {a, b} and outputs a list with three elements:
the first element is the new integrand (newintegrand) in terms of u (and
possibly still x); the second element is an equation giving u in terms of
x; the third element is a list of replacements of the new limits of
integration in terms of u."


s/: s[f_, {a_, b_}, x_, choice_:1] := 
      If[a <= x <= b, f[(1 - choice)*a + choice*b], 0] 


randompartition/: randompartition[{a_, b_}, n_] := P = 
   Union[Table[Random[Real, {N[a], N[b]}], {n - 1}], {N[a], N[b]}]

randompartition::usage = "randompartition[{a, b}, n] produces a random
partition of the interval {a, b} into at most n subintervals by randomly
picking n - 1 points in {a, b}. The partition is set equal to P."


regularpartition/: regularpartition[{a_, b_}, n_] := P = Block[{i},
    Table[N[a + i (b - a)/n], {i, 0, n}]]

regularpartition::usage = "regularpartition[{a, b}, n] partitions the
interval {a, b} into n equally spaced subintervals. The partition is set
equal to P."


refine/: refine[partition_, n_] := P =
    Union[Table[Random[Real, {Min[N[partition]], Max[N[partition]]}], {n}],
             N[partition]]

refine::usage = "refine[partition, n] randomly inserts n (possibly not
distinct) points into the given partition. The new partition is set equal
to P."


arbitraryriemann/: 
  arbitraryriemann[f_, P_, choice_:1] := Block[{i}, 
   Sum[(Union[N[P]][[i + 1]] - Union[N[P]][[i]])*
   f[(1 - choice)*Union[N[P]][[i]] + choice*Union[N[P]][[i + 1]]],
    {i, 1, Length[P] - 1}]]

arbitraryriemann::usage = "arbitraryriemann[f, P, choice] calculates the
Riemann sum of f for the partition P by evaluating f at the chosen point of
each subinterval of P. The chosen point is determined by a number between
0 and 1, ranging from the left end point to the right end point of each
subinterval. The default choice is 1, ie. choosing the right end point of
each subinterval " 
 

ear/: ear[f_, P_, choice_:1] := 
     Block[{x}, Abs[NIntegrate[f[x], {x, Min[N[P]], Max[N[P]]}] -
     arbitraryriemann[f, P, choice]]]

ear::usage = "ear[f, P, choice] calculates the difference between the
value of the defintite integral of f on the interval {Min[P], Max[P]} and
arbitraryriemann[f, P, choice]."


norm/: norm[P_] := Max[Drop[Union[N[P]], 1] - Drop[Union[N[P]], -1]]

norm::usage = "norm[P] is the norm of the partition P."

 
viewapprox/: viewapprox[f_, P_, choice_:1] := 
   (one = Block[{x}, Plot[f[x], {x, Min[N[P]] - 0.1, Max[N[P]] + 0.1},
           DisplayFunction -> Identity]];
    two = Block[{i, x},  Plot[Release[Table[s[f, {Union[N[P]][[i]],
         Union[N[P]][[i + 1]]}, x, choice], 
         {i, Length[P] - 1}]], {x, Min[N[P]] - 0.1, Max[N[P]] + 0.1},
      PlotRange -> All, PlotPoints -> 50, DisplayFunction -> Identity]];
      Block[{x}, Show[two, one, PlotLabel ->
         {f[x], (Length[P] - 1)*"intervals" , N[norm[P]]*"= norm", 
  If[Chop[ear[f, P, choice]] == 0, "no error", ear[f, P, choice]*"= error"]}, 
  PlotRange -> All, DisplayFunction -> $DisplayFunction]])

viewapprox::usage = "viewapprox[f, P, choice] shows the Riemann sum
approximation to the definite integral of f on the interval {Min[P],
Max[P]} in terms of the given partition P and the indicated choice of
point in each subinterval. The default choice is to evaluate f at the
right end point of each subinterval. The PlotLabel includes the function
f, the number of subintervals, the norm of the partition, and the absolute
error of the Riemann approximation. If P is a partition with some points
extremely close together - as may happen with a random partition - the
narrrow rectangles on these short subintervals may be missed in the output." 

  
average/: average[f_, {a_, b_}] := Integrate[f[x], {x, a, b}]/(b - a)
 
average::usage = "average[f, {a, b}] is the average value of f on the
interval {a, b}."


 
